@@ -2,17 +2,16 @@
 # Copyright (c) 2020, KarjaKAK
 # All rights reserved.
 
-import os
 import re
+import subprocess
 from pathlib import Path
-from subprocess import run
 from sys import platform
 
 import markdown
+import pdfkit
 
 __all__ = [""]
 
-
 extensions = [
     "pymdownx.extra",
     "pymdownx.caret",
@@ -32,7 +31,69 @@ extension_configs = {
 }
 
 
-def convhtml(text: str, filename: str, font: str, bg: str = None, fg: str = None):
+def _pattern(wrd: str, pat: str):
+    try:
+        last = g = fx = pt = None
+        if "<p>" not in wrd:
+            last = -5
+        else:
+            last = -4
+        g = wrd.partition(pat)
+        g = g[0] + g[1]
+        pt = (
+            '<input type="checkbox" class="strikethrough" name="ck"/><span for="ck">'
+            if not "checked" in pat
+            else '<input type="checkbox" class="strikethrough" name="ck" checked/><span for="ck">'
+        )
+        fx = (
+            wrd[: len(g)]
+            + "<span>"
+            + wrd[len(g) : last]
+            + "</span>"
+            + wrd[last:]
+            + "\n"
+        ).replace(
+            f"{pat}<span>",
+            pt,
+        )
+        return fx
+    finally:
+        del last, g, fx, pt, wrd, pat
+
+
+def _checking_wkhtmltopdf():
+    result = subprocess.run(
+        ["which", "wkhtmltopdf"],
+        stderr=subprocess.PIPE,
+        stdin=subprocess.PIPE,
+        stdout=subprocess.PIPE,
+        text=True,
+    )
+    if "wkhtmltopdf" in result.stdout:
+        return True
+    else:
+        return False
+
+
+def _save_pdf(scr: str, pdfpath: str):
+    options = {
+        "print-media-type": True,
+    }
+    pdfkit.from_file(scr, pdfpath, options=options)
+    del options
+
+
+def convhtml(
+    text: str,
+    filename: str,
+    font: str,
+    bg: str = None,
+    fg: str = None,
+    preview: bool = True,
+    width: int = 0,
+    height: int = 0,
+    pdfpath: str = None,
+):
     # Converting your TVG to html and printable directly from browser.
 
     try:
@@ -41,6 +102,10 @@ def convhtml(text: str, filename: str, font: str, bg: str = None, fg: str = None
         background = bg if bg else "gold"
         foreground = fg if fg else "black"
         kbfg = "333" if foreground == "black" else "eee"
+        startupinfo = None
+        window = None
+        pointer_event = None
+        # PROCS.collect()
 
         for i in gettext:
             if i != "\n":
@@ -98,10 +163,12 @@ list-style-type: none !important;
     text-decoration: line-through;
 }
 """
+        if preview:
+            pointer_event = """body { pointer-events: none; }"""
 
         cssstyle = f"""<!DOCTYPE html>
 <html>
-<button class="button"  onclick="javascript:window.print();">Print</button>
+<button class="button" onclick="javascript:window.print();">Print</button>
 <header>
 <meta charset="UTF-8">
 <h1>
@@ -114,6 +181,8 @@ list-style-type: none !important;
 {setfont}
 {kbd}
 {tasklist}
+{pointer_event if preview else ""}
+
 """
         printed = """@media print {
 .button { display: none; }
@@ -137,38 +206,10 @@ kbd { color: black !important; }
         fcs = []
         if 'input type="checkbox"' in cssstyle:
             for i in cssstyle.split("\n"):
-                if '<p><input type="checkbox"/>' in i:
-                    g = '<p><input type="checkbox"/>'
-                    fx = (
-                        i[: len(g)]
-                        + "<span>"
-                        + i[len(g) : -4]
-                        + "</span>"
-                        + i[-4:]
-                        + "\n"
-                    )
-                    fx = fx.replace(
-                        g + "<span>",
-                        '<p><input type="checkbox" class="strikethrough" name="ck"/><span for="ck">',
-                    )
-                    fcs.append(fx)
-                    del g, fx
-                elif '<p><input type="checkbox" checked/>' in i:
-                    g = '<p><input type="checkbox" checked/>'
-                    fx = (
-                        i[: len(g)]
-                        + "<span>"
-                        + i[len(g) : -4]
-                        + "</span>"
-                        + i[-4:]
-                        + "\n"
-                    )
-                    fx = fx.replace(
-                        g + "<span>",
-                        '<p><input type="checkbox" class="strikethrough" name="ck" checked/><span for="ck">',
-                    )
-                    fcs.append(fx)
-                    del g, fx
+                if '<input type="checkbox" checked/>' in i:
+                    fcs.append(_pattern(i, '<input type="checkbox" checked/>'))
+                elif '<input type="checkbox"/>' in i:
+                    fcs.append(_pattern(i, '<input type="checkbox"/>'))
                 else:
                     fcs.append(f"{i}\n")
 
@@ -179,16 +220,30 @@ kbd { color: black !important; }
             whtm.write(cssstyle)
         pro = None
         if platform.startswith("win"):
+            # ToDo: Preview for arm64 architecture
+            # In preview no interactive printing even it open on browser
             pro = [
                 "powershell.exe",
                 "start",
                 "msedge",
                 f"'\"{Path(f'{filename}.html').absolute()}\"'",
             ]
-            run(pro)
+            startupinfo = subprocess.STARTUPINFO()
+            startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
+            subprocess.run(pro, startupinfo=startupinfo)
         else:
-            pro = ["open", "-a", "Safari", f"{Path(f'{filename}.html').absolute()}"]
-            run(pro)
+            if preview or not _checking_wkhtmltopdf():
+                pro = [
+                    "open",
+                    f"{Path(f'{filename}.html').absolute()}",
+                ]
+                subprocess.run(pro)
+            else:
+                _save_pdf(f"{filename}.html", pdfpath=pdfpath)
+                subprocess.run(["open", f"{Path(pdfpath).name}"])
+    except Exception as e:
+        raise e
+    finally:
         del (
             text,
             filename,
@@ -206,6 +261,10 @@ kbd { color: black !important; }
             printed,
             nxt,
             pro,
+            startupinfo,
+            pointer_event,
+            window,
+            preview,
+            width,
+            height,
         )
-    except Exception as e:
-        raise e
